#!/bin/bash
#
# Usage: install.sh <imagefile>
#
# VERSION=6

rebootflag=$2
file=$1
image_dir=/tmp/images
mkdir -p $image_dir

function log
{
	echo ">> $1"
}

if [ -z "$file" ] ; then
	log "no filename given"
	exit 1
fi

if [ ! -f "${image_dir}/${file}" ] ; then
	log "file not found"
	exit 1
fi

log "getting VERSION file"
cd /tmp && tar zxf ${image_dir}/${file} VERSION
. /tmp/VERSION

log "type: $TYPE"
case "$TYPE" in
	fpga)
		log "unpacking fpga image"
		cd /tmp && tar zxf ${image_dir}/${file} fpga.img

		/sbin/insmod /usr/local/modules/bfpga.ko
		if [ -f /tmp/fpga.img ] ; then
			log "updating version file"
			mv /tmp/VERSION /usr/conf/VERSION.fpga

			log "installing fpga image, please stand by. This process may take 3 minutes. Do NOT turn off beroFix!"
			dd if=/tmp/fpga.img of=/dev/bfpga bs=512k count=1
		fi
	;;
	firmware)
		log "unpacking firmware"
		cd /tmp && tar zxf ${image_dir}/${file} firmware.axf

		log "updating firmware"
		dd if=/tmp/firmware.axf of=/dev/mtdblock3

		log "removing temporary file"
		rm /tmp/firmware.axf

		log "updating version file"
		mv /tmp/VERSION /usr/conf/VERSION.firmware
	;;
	kernel)
		log "unpacking kernel"
		cd /tmp && tar zxf ${image_dir}/${file} zImage

		log "updating kernel"
		dd if=/tmp/zImage of=/dev/mtdblock4

		log "removing temporary file"
		rm /tmp/zImage

		log "updating version file"
		mv /tmp/VERSION /usr/conf/VERSION.kernel
	;;
	rootfs)
		log "stopping services"
		/etc/init.d/S90app stop
		/etc/init.d/S60httpd stop

		log "loading berofix module"
		insmod /usr/local/modules/berofix.ko

		if [ -f /etc/ssh_host_key ]
		then
			log "copying ssh keys to conf partition"
			[ ! -d /usr/conf/ssh ] && mkdir /usr/conf/ssh
			cp -a /etc/ssh_host* /usr/conf/ssh/
		fi

		log "unpacking new rootfs"
		rm -rf /tmp/newroot
		mkdir /tmp/newroot || exit 1
		cd /tmp/newroot
		tar zxf ${image_dir}/${file} root.jffs2 || exit 1

		log "writing new rootfs to flash memory"
		dd if=root.jffs2 of=/dev/mtdblock5

		log "rebooting"
		dd if=/proc/RESET of=/dev/null
	;;
	userappfs)

		if [ -z ${NAME} ]; then
			log "[userapp] Name of this UserApp is empty, leaving."
			exit 1
		fi

		log "[userapp] Installing UserApp ${NAME}"
		log "[userapp] Description: ${DESCRIPTION}"

		log "[userapp] checking archive integrity"
		cd /tmp
		tar zxf ${image_dir}/${file} content.txt 2> /dev/null
		if [ ! -f /tmp/content.txt ]; then
			log "[userapp] FAILED: archive not compatible!"
			exit 1
		fi
		tar tzf ${image_dir}/${file}  > /tmp/compare-content.txt
		for i in $(cat /tmp/content.txt | sed "s/\/$//" )  ; do
			if ! grep "$i" compare-content.txt > /dev/null ; then
				log "[userapp] FAILED: file '${i}' missing in archive, leaving."
				exit 1
			fi
		done

		log "[userapp] checking versions"

		# check kernel
		if [ -f /usr/conf/VERSION.kernel ] ; then
			. /usr/conf/VERSION.kernel
		else
			KERNEL=1
		fi
		if [ ${NEED_KERNEL} -gt ${KERNEL} ] ; then
			log "[userapp] FAILED: please update kernel to version ${KERNEL} and try again!"
			exit 1
		fi

		# check rootfs
		if [ -f /VERSION ] ; then
			. /VERSION
		else
			ROOTFS=1
		fi
		if [ ${NEED_ROOTFS} -gt ${ROOTFS} ] ; then
			log "[userapp] FAILED: please update rootfs to version ${ROOTFS} and try again!"
			exit 1
		fi

		# check appfs
		if [ -f /usr/conf/VERSION.firmware ]; then
			. /usr/conf/VERSION.firmware
		else
			FIRMWARE=1
		fi
		if [ ${NEED_FIRMWARE} -gt ${FIRMWARE} ]; then
			log "[userapp] FAILED: please update firmware to version ${FIRMWARE} and try again!"
			exit 1
		fi

		log "[userapp] stopping services"
		/etc/init.d/S91userapp stop

		log "[userapp] mount userapp partition (rw)"
		mount -oremount,rw,user=admin /home/admin || exit 1

		log "[userapp] execute pre_remove script"
		[ -x /home/admin/setup/pre_remove.sh ] && /home/admin/setup/pre_remove.sh

		log "[userapp] remove version file"
		[ -f /home/admin/FILENAME ] && rm -f /home/admin/FILENAME

		log "[userapp] cleanup partition"
		rm -rf /home/admin/*

		log "[userapp] extract new image"
		cd /home/admin && tar zxf ${image_dir}/${file} || exit 1
		chown -R admin:admin /home/admin/*

		log "[userapp] execute post_install script"
		[ -x /home/admin/setup/post_install.sh ] && /home/admin/setup/post_install.sh

		log "[userapp] write version file"
		echo "${file}" > /home/admin/FILENAME

		log "[userapp] wait and sync"
		sleep 1; sync; sync; sleep 1

		log "[userapp] mount partition (ro)"
		mount -oremount,ro,user=admin /home/admin || exit 1

		log "[userapp] starting services"
		/etc/init.d/S91userapp start
		;;
	*)
                if ! echo $file | grep "downgrade" > /dev/null; then
		    log "checking for archive integrity"
                    cd /tmp && tar zxf ${image_dir}/${file} content.txt 2>/dev/null
		    if [ ! -f /tmp/content.txt ] ; then log "FAILED: Archive not compatible!" ; exit 1 ; fi
                    tar tzf ${image_dir}/${file}  > /tmp/compare-content.txt
                    for i in $(cat /tmp/content.txt | sed "s/\/$//" )  ; do
                      if ! grep "$i" compare-content.txt > /dev/null ; then log "FAILED: Missing Files in Archive!" ; exit 1; fi
                    done
                fi
                rm -rf /tmp/content.txt
                rm -rf /tmp/compare-content.txt

		log "checking versions"
		if [ -f /usr/conf/VERSION.kernel ] ; then . /usr/conf/VERSION.kernel ; else KERNEL=1 ; fi
		if [ -f /usr/conf/VERSION.firmware ] ; then . /usr/conf/VERSION.firmware ; else FIRMWARE=1 ; fi
		if [ -f /VERSION ] ; then . /VERSION ; else ROOTFS=1 ; fi

		if [ $NEED_KERNEL -gt $KERNEL ] ; then log "FAILED: please update kernel to version $KERNEL and try again!" ; exit 1 ; fi
		if [ $NEED_FIRMWARE -gt $FIRMWARE ] ; then log "FAILED: please update firmware to version $FIRMWARE and try again!" ; exit 1 ; fi
		if [ $NEED_ROOTFS -gt $ROOTFS ] ; then log "FAILED: please update rootfs to version $ROOTFS and try again!" ; exit 1 ; fi

		log "stopping services"
		/etc/init.d/S90app stop

		log "mount app partition (rw)"
		mount -o remount,rw /usr/local || exit 1

		log "execute pre_remove script"
		[ -x /usr/local/setup/pre_remove.sh ] && /usr/local/setup/pre_remove.sh

		log "remove version file"
		[ -f /usr/local/FILENAME ] && rm -f /usr/local/FILENAME

		log "cleanup app partition"
		cd /usr/local && ls | grep -v lost | xargs rm -rf

		log "extract new app image"
		cd /usr/local && tar zxf ${image_dir}/${file} || exit 1
		chown -R root:root /usr/local/*


		log "write version file"
		echo "$file" > /usr/local/FILENAME

		log "wait and sync"
		sleep 1
		sync ; sync
		sleep 1

                ${BEROCONF} set root app-status running
                ${BEROCONF} set root app-image `cat /usr/local/FILENAME`

                log "removing uploaded appfs"
                rm -f ${image_dir}/${file}
		if [ -f /home/admin/${file} ]; then
			mount -oremount,rw /home/admin
			rm -f /home/admin/${file}
			mount -oremount,ro /home/admin
		fi

		log "execute post_install script"
		[ -x /usr/local/setup/post_install.sh ] && /usr/local/setup/post_install.sh

		log "remount app partition (ro)"
		cd / && mount -o remount,ro /usr/local

                sync

#                if [ "$rebootflag" == "" ] ; then
#                    log "rebooting"
#                    sbin/reboot
#                fi

		log "start services"
		for i in /usr/local/init/S??* ; do
		        # Ignore dangling symlinks (if any).
		        [ ! -f "$i" ] && continue
		        if [ "$i" = "/usr/local/init/S99upgrade-firmware" ] || [ "$i" = "/usr/local/init/S02update-conf" ]; then
                       	echo "skipping service: $i"
	                        continue
        		fi
        		echo "starting service: $i"
	        	$i start
		done
                echo "Starting app services..."
		if [ -x /usr/local/bin/beroconf ] ; then
                        BEROCONF=/usr/local/bin/beroconf
                else
                        BEROCONF=/usr/fallback/beroconf
                fi

                /etc/init.d/S91userapp start
	;;
esac

log "done."

exit 0

